package com.dapr.service.controller;

import com.fasterxml.jackson.core.JsonProcessingException;
import com.fasterxml.jackson.databind.ObjectMapper;
import org.apache.http.HttpEntity;
import org.apache.http.HttpStatus;
import org.apache.http.StatusLine;
import org.apache.http.client.methods.CloseableHttpResponse;
import org.apache.http.client.methods.HttpPost;
import org.apache.http.entity.StringEntity;
import org.apache.http.impl.client.CloseableHttpClient;
import org.apache.http.impl.client.HttpClients;
import org.apache.http.util.EntityUtils;

import java.io.IOException;
import java.nio.charset.StandardCharsets;
import java.text.DateFormat;
import java.text.SimpleDateFormat;
import java.util.Calendar;
import java.util.Map;
import java.util.TimeZone;

public class CommonController {
    /**
     * Json serializer/deserializer.
     */
    protected static final ObjectMapper OBJECT_MAPPER = new ObjectMapper();

    /**
     * Format to output date and time.
     */
    protected static final DateFormat DATE_FORMAT = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss.SSS");

    protected String setBody(byte[] body, Map<String, String> headers) throws JsonProcessingException {

        String message = body == null ? "" : new String(body, StandardCharsets.UTF_8);

        Calendar utcNow = Calendar.getInstance(TimeZone.getTimeZone("GMT"));
        String utcNowAsString = DATE_FORMAT.format(utcNow.getTime());

        String metadataString = headers == null ? "" : OBJECT_MAPPER.writeValueAsString(headers);

        // Handles the request by printing message.
        System.out.println("Server: " + message);
        System.out.println("@ " + utcNowAsString + " and metadata: " + metadataString);
        return message;
    }

    /**
     * 请求mysql
     * curl -X POST -H 'Content-Type: application/json' http://localhost:3500/v1.0/bindings/talents-database-mysql-binding -d '
     * {
     *   "operation": "exec",
     *   "metadata": {
     *     "sql": "INSERT INTO t_award (resume_id, award_institution, award_name) VALUES (1, \"test111\", \"test112\")"
     *   }
     * }
     */
    protected void doMysqlPost(String sql) throws IOException {
        String  httpPort = "3501";

        String resultUrl = "http://121.43.155.15:" + httpPort + "/v1.0/bindings/mysql-binding";

        doPost(resultUrl, sql);
    }

    /**
     * 发查看存储的消息
     * @throws IOException
     */
    protected static void sendQueryMsg(String msg) throws IOException {

        //拼接SQL
        String bodyTmpl = "{\"msg\": \\\"%s\\\")\"}";
        String sql =String.format(bodyTmpl, "请求了查看");

        // 请求mq
        doMqPost("query-topic", msg);
    }

    /**
     * 发操作存储的消息
     * @param msg
     * @throws IOException
     */
    protected static void sendExecMsg(String msg) throws IOException {

        //拼接SQL
      //  String bodyTmpl = "{\"msg\": \\\"%s\\\"}";
     //   String sql =String.format(bodyTmpl, msg);

        // 请求mq
        doMqPost("exec-topic", msg);
    }

    /**
     * 请求mq
     * curl -X POST -H 'Content-Type: application/json' http://localhost:3500/v1.0/bindings/talents-database-mysql-binding -d '
     * {
     *   "operation": "exec",
     *   "metadata": {
     *     "sql": "INSERT INTO t_award (resume_id, award_institution, award_name) VALUES (1, \"test111\", \"test112\")"
     *   }
     * }
     */
    private static void doMqPost(String topic, String sql) throws IOException {
        String  httpPort = "3700";

        String resultUrl = "http://192.168.133.129:" + httpPort + "/v1.0/publish/rabbitmq-pubsub/"+ topic;

       doPost(resultUrl, sql);
    }

    /**
     * 统一发请求
     * @param resultUrl
     * @param sql
     * @return
     * @throws IOException
     */
    private static String doPost(String resultUrl, String sql) throws IOException {

        // 通过httpclient发送post请求
        CloseableHttpClient httpclient = HttpClients.createDefault();

        HttpPost httpPost = new HttpPost(resultUrl);// 创建httpPost

        httpPost.setHeader("Accept", "application/json");
        httpPost.setHeader("Content-Type", "application/json");

        StringEntity entity = new StringEntity(sql, "utf-8");

        System.out.println("Sql: " + sql);

        httpPost.setEntity(entity);

        CloseableHttpResponse response = null;

        try {
            response = httpclient.execute(httpPost);

            StatusLine status = response.getStatusLine();
            int state = status.getStatusCode();
            // if (state == HttpStatus.SC_OK) {
            if (state == HttpStatus.SC_NO_CONTENT) {
                HttpEntity responseEntity = response.getEntity();
                String respContent = EntityUtils.toString(responseEntity,"UTF-8");

              //  System.out.println("成功返回:" + state + "结果：" + respContent);

                System.out.println("成功返回:" + state + "(" + response.toString() + ")");

                return respContent;
            } else {
                System.out.println("请求返回:" + state + "(" + response.toString() + ")");
            }
        } finally {
            if (response != null) {
                try {
                    response.close();
                } catch (IOException e) {
                    e.printStackTrace();
                }
            }

            try {
                httpclient.close();
            } catch (IOException e) {
                e.printStackTrace();
            }
        }
        return null;
    }


}
